@@ -28,6 +28,11 @@ import geo
 from threading import Thread
 import threading
 import urllib3
+import random
+#import traceback
+import modrana_utils
+
+
 
 import socket
 timeout = 30 # this sets timeout for all sockets
@@ -35,6 +40,21 @@ socket.setdefaulttimeout(timeout)
 
 
 
+class TileNotImageException(Exception):
+       def __init__(self):
+           self.parameter = 1
+       def __str__(self):
+
+         message = "the downloaded tile is not an image as per \
+its magic number (it is probably an error response webpage \
+returned by the server)"
+#         message = "the downloaded tile is not an image as per "
+#         message+= "its magic number (it is probably an error response webpage "
+#         message+= "returned by the server)"
+
+         return message
+         
+
 #from modules.pyrender.tilenames import xy2latlon
 import sys
 if(__name__ == '__main__'):
@@ -63,6 +83,9 @@ class mapData(ranaModule):
     self.notificateOnce = True
     self.scroll = 0
     self.onlineRequestTimeout = 30
+    """ how often should be the download info
+    updated during batch, in seconds"""
+    self.batchInfoUpdateInterval = 0.5
 
 
   def listTiles(self, route):
@@ -446,7 +469,7 @@ class mapData(ranaModule):
         threads.append(t)
       print "Added %d URLS to check for size." % self.urlCount
       while True:
-        time.sleep(0.5) # this governs how often we check status of the worker threads
+        time.sleep(self.callback.batchInfoUpdateInterval) # this governs how often we check status of the worker threads
         print "Batch size working...",
         print "(threads: %i)," % (threading.activeCount()-1, ),
         print"pending: %d, done: %d" % (len(self.neededTiles),self.processed)
@@ -456,10 +479,10 @@ class mapData(ranaModule):
           self.finished = True
           break
         if not self.neededTiles: # we processed everything
-          print "***get size finished"
-          shutdown.set()
-          self.finished = True
-          break
+              print "***get size finished"
+              shutdown.set()
+              self.finished = True
+              break
 
     def getSizeWorker(self, shutdown, incrementLock,localDlListLock):
       """a worker thread method for estimating tile size"""
@@ -529,14 +552,49 @@ class mapData(ranaModule):
       self.neededTiles = neededTiles
       self.maxThreads = maxThreads
       self.layer=layer
-      self.processed = 0
-      self.transfered = 0
+
+      self.retryCount = 3
+
+      self.retryInProgress = 0
+
       self.urlCount = len(neededTiles)
       self.finished = False
       self.quit = False
       url = self.getAnUrl(neededTiles)
       self.connPool = self.createConnectionPool(url)
 
+      # only access following variables with the incrementLock
+      self.processed = 0 # counter for processed tiles
+      self.found = 0 # number of tiles found locally
+      self.downloaded = 0 # counter for downloaded tiles
+      self.failedDownloads = []
+      self.transfered = 0
+
+    def _resetCounts(self):
+      self.processed = 0
+      self.urlCount = len(self.neededTiles)
+      self.failedDownloads = []
+
+    def getProgress(self):
+      return (self.processed, self.urlCount, self.transfered, self.getFailedDownloadCount())
+
+    def getFailedDownloads(self):
+      return self.failedDownloads
+    
+    def getFailedDownloadCount(self):
+      return len(self.failedDownloads)
+
+    def getDownloadCount(self):
+      return self.downloaded
+
+    def getRetryInProgress(self):
+      """return 0 if normal download is in progress
+      or 1-n for 1 - n-th retry"""
+      return self.retryInProgress
+
+    def isFinished(self):
+      return self.finished
+
     def getAnUrl(self, neededTiles):
       """get a random url so we can init the pool"""
       if neededTiles:
@@ -549,8 +607,7 @@ class mapData(ranaModule):
       else:
         url = ""
       return url
-
-
+    
     def createConnectionPool(self, url):
       """create the connection pool -> to facilitate socket reuse"""
       timeout = self.callback.onlineRequestTimeout
@@ -562,6 +619,7 @@ class mapData(ranaModule):
       maxThreads = self.maxThreads
       shutdown = threading.Event()
       incrementLock = threading.Lock()
+      workFinished = False
 
       threads = []
       for i in range(0,maxThreads): # start threads
@@ -574,7 +632,7 @@ class mapData(ranaModule):
       print "minipool initialized"
       print "Added %d URLS to download." % self.urlCount
       while True:
-        time.sleep(0.5)
+        time.sleep(self.callback.batchInfoUpdateInterval)
         print "Batch tile dl working...",
         print"(threads: %i)" % (threading.activeCount()-1, ),
         print"pending: %d, done: %d" % (len(self.neededTiles),self.processed)
@@ -586,15 +644,30 @@ class mapData(ranaModule):
           self.finished = True
           break
         if not self.neededTiles: # we processed everything
-          print "***get tiles finished"          
+          if self.failedDownloads: # retry failed downloads
+            if self.retryCount > 0:
+              # retry failed tiles
+              self.retryCount-=1
+              self.retryInProgress+=1
+              self.neededTiles = list(self.failedDownloads) # make a copy
+              self._resetCounts()
+            else: # retries exhausted, quit
+              workFinished = True
+
+          else: # no failed downloads, just quit
+            workFinished = True
+
+        if workFinished: # check if we are done
+          print "***get tiles finished"
           if self.callback.notificateOnce:
             self.callback.sendMessage('ml:notification:m:Batch download complete.;7')
             self.callback.notificateOnce = False
-
           shutdown.set()
           self.finished = True
           break
 
+
+
     def getTilesWorker(self, shutdown, incrementLock):
       while 1:
         if shutdown.isSet():
@@ -610,14 +683,29 @@ class mapData(ranaModule):
           continue
           
         # try to retrieve and store the tile
+        failed = False
+        dl = False
         try:
-          self.saveTileForURL(item)
+
+ #TESTING ONLY ! - random error generator
+#          r = random.randint(1, 6)
+#          if r == 6:
+#            "BOOM"/2
+
+          dlSize = self.saveTileForURL(item)
         except Exception, e:
+          failed = True
           # TODO: try to redownload failed tiles
           print "exception in get tiles thread:\n%s" % e
+#          traceback.print_exc(file=sys.stdout) # find what went wrong
         # increment the counter in a thread safe way
         with incrementLock:
           self.processed+=1
+          if failed:
+            self.failedDownloads.append(item)
+          elif dlSize!=False:
+            self.downloaded+=1
+            self.transfered+=dlSize
 
     def saveTileForURL(self, tile):
       """save a tile for url created from its coordinates"""
@@ -629,8 +717,27 @@ class mapData(ranaModule):
         # TODO: maybe make something like tile objects so we dont have to pass so many parameters ?
         if not m.tileExists(filename, folderPrefix, z, x, y, layerType, fromThread = True): # if the file does not exist
           request = self.connPool.get_url(url)
+          size = int(request.getheaders()['content-length'])
           content = request.data
-          m.automaticStoreTile(content, folderPrefix, z, x, y, layerType, filename, folder, fromThread = True)
+          """
+          The tileserver sometimes returns a HTML error page
+          instead of the tile, which is then saved instead of the tile an
+          users are then confused why tiles they have downloaded don't show up.
+
+          To raise a proper error on this behaviour, we check the tiles magic number
+          and if is not an image we raise the TileNotImageException.
+
+          TODO: does someone supply nonbitmap/SVG tiles ?
+          """
+          if modrana_utils.isTheStringAnImage(content):
+            #its an image, save it
+            m.automaticStoreTile(content, folderPrefix, z, x, y, layerType, filename, folder, fromThread = True)
+          else:
+            # its not ana image, raise exception
+            raise TileNotImageException()
+          return size # something was actually downloaded and saved
+        else:
+          return False # nothing was downloaded
 
   def expand(self, tileset, amount=1):
     """Given a list of tiles, expand the coverage around those tiles"""
@@ -769,7 +876,10 @@ class mapData(ranaModule):
       menus.drawButton(cr, (x1+w)-2*dx, y1, dx, dy, "edit", "tools", "menu:setupEditBatchMenu|set:menu:editBatch")
       # * draw "start" button
       if self.getFilesThread:
-        menus.drawButton(cr, (x1+w)-1*dx, y1, dx, dy, "stop", "stop", "mapData:stopDownloadThreads")
+        if self.getFilesThread.isFinished():
+          menus.drawButton(cr, (x1+w)-1*dx, y1, dx, dy, "retry", "start", "mapData:download")
+        else:
+          menus.drawButton(cr, (x1+w)-1*dx, y1, dx, dy, "stop", "stop", "mapData:stopDownloadThreads")
       else:
         menus.drawButton(cr, (x1+w)-1*dx, y1, dx, dy, "start", "start", "mapData:download")
       # * draw the combined info area and size button (aka "box")
@@ -783,21 +893,21 @@ class mapData(ranaModule):
         menus.drawButton(cr, boxX, boxY, boxW, boxH, "", "generic", "mapData:getSize")
       # * display information about download status
       getFilesText = self.getFilesText(getFilesThread)
-      getFilesTextX = boxX + dx/8
-      getFilesTextY = boxY + boxH*1/4
-      self.showText(cr, getFilesText, getFilesTextX, getFilesTextY, w-dx/4, 40)
-
-      # * display information about size of the tiles
       sizeText = self.getSizeText(sizeThread)
-      sizeTextX = boxX + dx/8
-      sizeTextY = boxY + boxH*2/4
-      self.showText(cr, sizeText, sizeTextX, sizeTextY, w-dx/4, 40)
+      getFilesTextX = boxX + dx/8
+      getFilesTextY = boxY + boxH*1/10
+      menus.showText(cr, "%s\n\n%s" % (getFilesText, sizeText), getFilesTextX, getFilesTextY, w-dx/4, 40)
 
-      # * display information about free space available (for the filesystem with the tilefolder)
+#      # * display information about size of the tiles
+#      sizeTextX = boxX + dx/8
+#      sizeTextY = boxY + boxH*2/4
+#      menus.showText(cr, sizeText, sizeTextX, sizeTextY, w-dx/4, 40)
+
+      # * display information about free space available (for the filesystem with tilefolder)
       freeSpaceText = self.getFreeSpaceText()
       freeSpaceTextX = boxX + dx/8
       freeSpaceTextY = boxY + boxH * 3/4
-      self.showText(cr, freeSpaceText, freeSpaceTextX, freeSpaceTextY, w-dx/4, 40)
+      menus.showText(cr, freeSpaceText, freeSpaceTextX, freeSpaceTextY, w-dx/4, 40)
 
     if menuName == 'chooseRouteForDl':
 
@@ -839,19 +949,39 @@ class mapData(ranaModule):
 
   def getFilesText(self, getFilesThread):
     """return a string describing status of the download threads"""
+    text = ""
     tileCount = len(self.currentDownloadList)
-    if tileCount == 0:
-      return "All tiles for this area are available."
-    elif getFilesThread == None:
-      return ( "Press Start to download ~ %d tiles." % tileCount)
-    elif getFilesThread.isAlive() == True:
-      totalTileCount = getFilesThread.urlCount
-      currentTileCount = getFilesThread.processed
-      text = "Downloading: %d of %d tiles complete" % (currentTileCount, totalTileCount)
-      return text
-    elif getFilesThread.isAlive() == False: #TODO: send an alert that download is complete
-      text = "Download complete."
-      return text
+    if getFilesThread == None:
+      if tileCount:
+        text = "Press <b>Start</b> to download ~ <b>%d</b> tiles." % tileCount
+      else:
+        text = "Download queue empty."
+    else:
+      (currentTileCount, totalTileCount, BTotalTransfered, failedCount) = getFilesThread.getProgress()
+      if getFilesThread.isAlive() == True:
+        MBTotalTransfered = BTotalTransfered/float(1048576)
+        totalTileCount = getFilesThread.urlCount
+        currentTileCount = getFilesThread.processed
+        retryNumber = getFilesThread.getRetryInProgress()
+        if retryNumber:
+          action = "Retry nr. %d" % retryNumber
+        else:
+          action = "Downloading"
+          
+        text = "<b>%s</b>: <b>%d</b> of <b>%d</b> tiles done\n\n<b>%1.2f MB</b> transfered, %d downloads failed" % (action, currentTileCount, totalTileCount, MBTotalTransfered, failedCount)
+      elif getFilesThread.isAlive() == False: #TODO: send an alert that download is complete
+        if getFilesThread.getDownloadCount():
+          # some downloads occured
+          text = "<b>Download complete.</b>"
+        else:
+          # no downloads occured
+          if failedCount:
+            # no downloads + failed downloads
+            text = "<b>Download of all tiles failed.</b>"
+          else:
+            # no downalods and no failed downloads
+            text = "<b>All tiles were locally available.</b>"
+    return text
 
   def getSizeText(self, sizeThread):
     """return a string describing status of the size counting threads"""
@@ -859,12 +989,12 @@ class mapData(ranaModule):
     if tileCount == 0:
       return ""
     if sizeThread == None:
-      return ("Total size of tiles is unknown (click to compute).")
+      return ("Total size of tiles is unknown (<i>click to compute</i>).")
     elif sizeThread.isAlive() == True:
       totalTileCount = sizeThread.urlCount
       currentTileCount = sizeThread.processed
       currentSize = sizeThread.totalSize/(1048576) # = 1024.0*1024.0
-      text = "Checking: %d of %d tiles complete(%1.0f MB)" % (currentTileCount, totalTileCount, currentSize)
+      text = "Checking: %d of %d tiles complete(<b>%1.0f MB</b>)" % (currentTileCount, totalTileCount, currentSize)
       return text
     elif sizeThread.isAlive() == False:
       sizeInMB = sizeThread.totalSize/(1024.0*1024.0)
@@ -880,72 +1010,6 @@ class mapData(ranaModule):
     text = "Free space available: %1.1f MB" % freeSpaceInMB
     return text
 
-
-  def showText(self,cr,text,x,y,widthLimit=None,fontsize=40):
-    if(text):
-      cr.set_font_size(fontsize)
-      stats = cr.text_extents(text)
-      (textwidth, textheight) = stats[2:4]
-
-      if(widthLimit and textwidth > widthLimit):
-        cr.set_font_size(fontsize * widthLimit / textwidth)
-        stats = cr.text_extents(text)
-        (textwidth, textheight) = stats[2:4]
-
-      cr.move_to(x, y+textheight)
-      cr.show_text(text)
-
-  def sendMessage(self,message):
-    m = self.m.get("messages", None)
-    if(m != None):
-      print "mapData: Sending message: " + message
-      m.routeMessage(message)
-    else:
-      print "mapData: No message handler, cant send message."
-
-  def tilesetSvgSnippet(self, f, tileset, colour):
-    for tile in tileset:
-      (x,y) = [int(a) for a in tile.split(",")]
-      f.write("<rect width=\"1\" height=\"1\" x=\"%d\" y=\"%d\" style=\"fill:%s;stroke:#000000;stroke-width:0.05;\" id=\"rect2160\" />\n" % (x,y, colour))
-  
-  def routeSvgSnippet(self, f, route):
-    path = None
-    for pos in route:
-      (lat,lon) = pos
-      (x,y) = latlon2xy(lat, lon, 15)
-      if(path == None):
-        path = "M %f,%f" % (x,y)
-      else:
-        path += " L %f,%f" % (x,y)
-
-    f.write("<path       style=\"fill:none; stroke:white; stroke-width:0.12px;\" d=\"%s\"        id=\"inner\" />\n" % path)
-
-    f.write("<path       style=\"fill:none; stroke:yellow; stroke-width:0.06px;\" d=\"%s\"        id=\"outer\" />\n" % path)
-      
-
-  def tilesetToSvg(self, tilesets, route, filename):
-    f = open(filename, "w")
-    f.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n")
-    f.write("<svg\n   xmlns:svg=\"http://www.w3.org/2000/svg\"\n   xmlns=\"http://www.w3.org/2000/svg\"\n   version=\"1.0\"\n   width=\"1000\"\n   height=\"1000\"   id=\"svg2\">\n")
-
-    print "Creating SVG"
-    f.write("  <g id=\"layer1\">\n")
-    colours = ['red','#FF8000','yellow','green','blue','#808080','black']
-    for tileset in tilesets:
-      colour = colours.pop(0)
-      print " - tileset %s"% colour
-      self.tilesetSvgSnippet(f,tileset, colour)
-    f.write("</g>\n")
-    
-    if(route):
-      f.write("  <g id=\"route\">\n")
-      print " - route"
-      self.routeSvgSnippet(f, route)
-      f.write("</g>\n")
-     
-    f.write("</svg>\n")
-
-
   def stopSizeThreads(self):
     if self.sizeThread:
       try:
@@ -969,56 +1033,3 @@ class mapData(ranaModule):
   def shutdown(self):
     self.stopSizeThreads()
     self.stopBatchDownloadThreads()
-
-
-
-
-if(__name__ == "__main__"):
-  from sample_route import *
-  route = getSampleRoute()
-  a = mapData({}, {})
-
-  if(0): # spirals
-    for d in range(1,10):
-      # Create a spiral of tiles, radius d
-      tileset = a.spiral(100,100,0, d)
-      print "Spiral of width %d = %d locations" %(d,len(tileset))
-
-      # Convert array to dictionary
-      keys = {}
-      count = 0
-      for tile in tileset:
-        (x,y,z) = tile
-        key = "%d,%d" % (x,y)
-        keys[key] = " " * (5-len(str(count))) + str(count)
-        count += 1
-
-      # Print grid of values to a textfile
-      f = open("tiles_%d.txt" % d, "w")
-      for y in range(100 - d - 2, 100 + d + 2):
-        for x in range(100 - d - 2, 100 + d + 2):
-          key = "%d,%d" % (x,y)
-          val = keys.get(key, " ")
-          f.write("%s\t" % val)
-        f.write("\n")
-      f.close()
-          
-  # Load a sample route, and try expanding it
-  if(1):
-    tileset = a.listTiles(route)
-    from time import time
-    print "Route covers %d tiles" % len(tileset)
-
-
-    a.tilesetToSvg([tileset], route, "route.svg")
-       
-    if(1):
-      tilesets = []
-      for i in range(0,14,2):
-        start = time()
-        result = a.expand(tileset,i)
-        dt = time() - start
-        print " expand by %d to get %d tiles (took %1.3f sec)" % (i,len(result), dt)
-        tilesets.insert(0,result)
-
-      a.tilesetToSvg(tilesets, route, "expand.svg" )
