@@ -31,14 +31,28 @@ class mapView(ranaModule):
   def firstTime(self):
     self.checkMapDraggingMode() # check the map dragging mode on startup
     self.checkCenteringDisableTreshold() # check centering disable treshold on startup
+    self.lastZ = int(self.get('z', 15))
     
   def handleMessage(self, message, type, args):
     z = self.get('z', 15)
     if(message == 'zoomIn'):
-      self.set('z', z + 1)
+      newZ = z + 1
+      self.set('z', newZ)
+      proj = self.m.get('projection', None)
+      if proj:
+        proj.setZoom(newZ)
+      self.notify("zooming <b>in</b> to zl %d" % newZ)
     elif(message == 'zoomOut'):
-#      self.set('z', max(z - 1, 8))
-      self.set('z', max(z - 1, 6))
+      newZ = max(z - 1, 4)
+      self.set('z', newZ)
+      proj = self.m.get('projection', None)
+      if proj:
+        proj.setZoom(newZ)
+      if newZ != z:
+        self.notify("zooming <b>out</b> to zl %d" % newZ)
+      else:
+        self.notify("minimum zoomlevel reached")
+
 
     elif (message=='recentreToPos'):
       pos = self.get('pos', None)
@@ -78,12 +92,11 @@ class mapView(ranaModule):
 
   def handleCentring(self):
     # check if centring is on
-    if(self.get("centred",True) or self.get('centreOnce', False)):
+    if self.get("centred",True):
       # get current position information
       pos = self.get('pos', None)
       # check if the position changed from last time
-      if(self.setCentre(pos)):
-        self.set('centreOnce', False)        
+      self.setCentre(pos)     
       
   def setCentre(self,pos):
     """takes care for centering the map on current position"""
@@ -104,8 +117,11 @@ class mapView(ranaModule):
       1 -> directly on the edge
       0.5 -> shifted by half of this distance, eq. in 3/4 of the screen
       """
-      proj.recentre(lat,lon,z)
-      self.set("needRedraw", True)
+      newZoom = None
+      if z != self.lastZ:
+        newZoom = z
+        self.lastZ = newZoom
+      proj.recentre(lat,lon,newZoom)
       return(True)
 
     # * map draging mode control * #
