@@ -19,11 +19,9 @@
 #---------------------------------------------------------------------------
 from base_module import ranaModule
 import pycha.line
-#from lines import lines
-import sys
+import pycha.color
 import cairo
 import geo
-import gtk
 
 def getModule(m,d,i):
   return(routeProfile(m,d,i))
@@ -56,6 +54,11 @@ class routeProfile(ranaModule):
     dx = min(w,h) / 5.0
     dy = dx
 
+    # draw a solid color background    
+    cr.rectangle(x1,y1,w,h)
+    cr.set_source_rgb(*pycha.color.hex2rgb("#eeeeff"))
+    cr.fill()
+
     menus = self.m.get("menu",None)
     loadTl = self.m.get('loadTracklogs', None) # get the tracklog module
     tracklog = loadTl.getActiveTracklog()
@@ -125,7 +128,7 @@ class routeProfile(ranaModule):
     if units == None:
       xTicks = [dict(v=r, label=list[r][0]) for r in range(0,length,labelTick)]
     else:
-      xTicks = [dict(v=r, label=units.km2CurrentUnitString(list[r][0])) for r in range(0,length,labelTick)]
+      xTicks = [dict(v=r, label=units.km2CurrentUnitString(round(list[r][0], 1))) for r in range(0,length,labelTick)]
 
 #    list = tracklog.trackpointsList[0]
 #    lines = tuple(map(lambda x: ("", float(x.elevation)), list))
